@@ -83,7 +83,7 @@ pub struct Btf {
     header: btf_header,
     strings: Vec<u8>,
     types: Vec<BtfType>,
-    endianness: Endianness,
+    _endianness: Endianness,
 }
 
 impl Btf {
@@ -110,7 +110,7 @@ impl Btf {
         }
 
         // safety: btf_header is POD so read_unaligned is safe
-        let header = unsafe { ptr::read_unaligned(data.as_ptr() as *const btf_header) };
+        let header = unsafe { read_btf_header(data) };
 
         let str_off = header.hdr_len as usize + header.str_off as usize;
         let str_len = header.str_len as usize;
@@ -125,7 +125,7 @@ impl Btf {
             header,
             strings,
             types,
-            endianness,
+            _endianness: endianness,
         })
     }
 
@@ -152,7 +152,6 @@ impl Btf {
             data = &data[ty.type_info_size()..];
             types.push(ty);
         }
-
         Ok(types)
     }
 
@@ -230,7 +229,12 @@ impl Btf {
 
             use BtfType::*;
             let size = match ty {
-                Int(ty, _) | Struct(ty, _) | Union(ty, _) | Enum(ty, _) | DataSec(ty, _) => {
+                Int(ty, _)
+                | Struct(ty, _)
+                | Union(ty, _)
+                | Enum(ty, _)
+                | DataSec(ty, _)
+                | Float(ty) => {
                     // Safety: union
                     unsafe { ty.__bindgen_anon_1.size as usize }
                 }
@@ -259,14 +263,19 @@ impl Btf {
     }
 }
 
+unsafe fn read_btf_header(data: &[u8]) -> btf_header {
+    // safety: btf_header is POD so read_unaligned is safe
+    ptr::read_unaligned(data.as_ptr() as *const btf_header)
+}
+
 #[derive(Debug, Clone)]
 pub struct BtfExt {
     data: Vec<u8>,
-    endianness: Endianness,
+    _endianness: Endianness,
     relocations: Vec<(u32, Vec<Relocation>)>,
     header: btf_ext_header,
-    func_info_rec_size: usize,
-    line_info_rec_size: usize,
+    _func_info_rec_size: usize,
+    _line_info_rec_size: usize,
     core_relo_rec_size: usize,
 }
 
@@ -313,11 +322,11 @@ impl BtfExt {
         let mut ext = BtfExt {
             header,
             relocations: Vec::new(),
-            func_info_rec_size: rec_size(func_info_off, func_info_len)?,
-            line_info_rec_size: rec_size(line_info_off, line_info_len)?,
+            _func_info_rec_size: rec_size(func_info_off, func_info_len)?,
+            _line_info_rec_size: rec_size(line_info_off, line_info_len)?,
             core_relo_rec_size: rec_size(core_relo_off, core_relo_len)?,
             data: data.to_vec(),
-            endianness,
+            _endianness: endianness,
         };
 
         let rec_size = ext.core_relo_rec_size;
@@ -400,7 +409,7 @@ impl<'a> Iterator for SecInfoIter<'a> {
 
         Some(SecInfo {
             sec_name_off,
-            num_info,
+            _num_info: num_info,
             data,
         })
     }
@@ -409,6 +418,28 @@ impl<'a> Iterator for SecInfoIter<'a> {
 #[derive(Debug)]
 pub(crate) struct SecInfo<'a> {
     sec_name_off: u32,
-    num_info: u32,
+    _num_info: u32,
     data: &'a [u8],
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    #[test]
+    fn test_parse_header() {
+        let data: &[u8] = &[
+            0x9f, 0xeb, 0x01, 0x00, 0x18, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x64, 0x54,
+            0x2a, 0x00, 0x64, 0x54, 0x2a, 0x00, 0x10, 0x64, 0x1c, 0x00,
+        ];
+        let header = unsafe { read_btf_header(data) };
+        assert_eq!(header.magic, 0xeb9f);
+        assert_eq!(header.version, 0x01);
+        assert_eq!(header.flags, 0x00);
+        assert_eq!(header.hdr_len, 0x18);
+        assert_eq!(header.type_off, 0x00);
+        assert_eq!(header.type_len, 0x2a5464);
+        assert_eq!(header.str_off, 0x2a5464);
+        assert_eq!(header.str_len, 0x1c6410);
+    }
+}
